/*===========================================================================*\
 *                                                                           *
 *                            ACG Localizer                                  *
 *      Copyright (C) 2011 by Computer Graphics Group, RWTH Aachen           *
 *                           www.rwth-graphics.de                            *
 *                                                                           *
 *---------------------------------------------------------------------------* 
 *  This file is part of ACG Localizer                                       *
 *                                                                           *
 *  ACG Localizer is free software: you can redistribute it and/or modify    *
 *  it under the terms of the GNU General Public License as published by     *
 *  the Free Software Foundation, either version 3 of the License, or        *
 *  (at your option) any later version.                                      *
 *                                                                           *
 *  ACG Localizer is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *  GNU General Public License for more details.                             *
 *                                                                           *
 *  You should have received a copy of the GNU General Public License        *
 *  along with ACG Localizer.  If not, see <http://www.gnu.org/licenses/>.   *
 *                                                                           *
\*===========================================================================*/ 

#ifndef SIFT_LOADER_HH
#define SIFT_LOADER_HH

/**
 *    Class to read SIFT features from the text files generated by
 *    David Lowe's binary ( available at http://www.cs.ubc.ca/~lowe/keypoints/).
 *
 *    IMPORTANT NOTE: The SIFT loader keeps the coordinate systems of the binaries. For example,
 *    the origin of the coordinate system used by David Lowe is in the upper left of the image.
 *  
 *  author : Torsten Sattler (tsattler@cs.rwth-aachen.de)
 *  date : 09-26-2011
**/ 

#include <vector>
#include <stdint.h>
#include "SIFT_keypoint.hh"


class SIFT_loader
{
  public:
    //! constructor
    SIFT_loader( );
    
    //! destructor
    ~SIFT_loader( );
	
    //! load features from a file with a given format
    void load_features( const char *filename, SIFT_FORMAT format );
	
    //! save the features loaded to a file in David Lowes Format
    bool save_features_lowe( const char *filename );
    
    //! clears all data loaded so far
    void clear_data( );
	
    //! returns the number of features loaded
    uint32_t get_nb_features( );
    
    //! get access to a vector containg the descriptors (stored as chars)
    std::vector< unsigned char* >& get_descriptors( );
    
    //! get the keypoints
    std::vector< SIFT_keypoint >& get_keypoints( );
	
  private:
    
    std::vector< SIFT_keypoint > mKeypoints;
    std::vector< unsigned char* > mDescriptors;
    uint32_t mNbFeatures;
    
    bool load_Lowe_features( const char *filename );
  
};


#endif